import "Simbo.NeoKhuzdul.PhrasesListBox";

NeoKhuzdulWindow = class(WindowLotro);

local Constants =
{
	Padding =
	{
		Top = 40,
		Left = 10,
		Right = 10,
		Bottom = 20,
		Lines = 6
	},
	Sizes =
	{
		Categories = 180,
		PhraseBox = 48,
		ReadButton = 32,
		GenderIcon = 20,
		TextBox = 200,
		CheckBox = 200,
		SortLabel = 75,
		Dropdown = 
		{
			Width = 140,
			Height = 20,
			Open = 135
		},
		SortIcon =
		{
			Width = 15,
			Height = 15
		}
	},
	MinimumSize =
	{
		Width = UIOptions["minimum_width"];
		Height = 420
	}
}

function NeoKhuzdulWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.SelectedCategory = nil;
	self.SelectedGender = nil;
	self.SelectedPhrase = nil;
	self.SelectedPhraseText = "";
	self.PlaceHolderText = "";
	self.MoreInfoIsShown = false;
	self.SelectedButton = nil;
	self.DisplayedPhrases = {};
	self.SortOrder = nil;
	
	self:SetText(Strings["neokhuzdul"]);
	self:SetPosition(tonumber(Settings.NeoKhuzdulWindow.Position.Left), tonumber(Settings.NeoKhuzdulWindow.Position.Top));
	
	self:SetSize(tonumber(Settings.NeoKhuzdulWindow.Size.Width), tonumber(Settings.NeoKhuzdulWindow.Size.Height));
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self:SetOpacity(tonumber(Settings.NeoKhuzdulWindow.WindowOpacity) / 100);
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	local height = 0;
	
	local padding = UIOptions["button_padding"];
	left = left - padding;
	
	for index, categoryData in ipairs(NeoKhuzdulDB.Categories) do 
		local button = CategoryButton(self, categoryData.Key, categoryData.Image, categoryData[LANGUAGE] or categoryData["EN"], padding);
		button:SetPosition(left, top);
		button.Click = function(sender, args)
			if self.SelectedCategory ~= sender.Category then
				if self.SelectedButton ~= nil then
					self.SelectedButton:SetSelected(false)
					self.SelectedButton = sender;
				end
				self.SelectedButton = sender;
				
				self.SelectedCategory = sender.Category;
				self.SelectedButton:SetSelected(true);
				self:PopulatePhrases();
			end
		end
		left = left + button:GetWidth() + (padding * 2);
		height = button:GetHeight();
	end
	
	top = top + height + Constants.Padding.Lines * 2;
	left = Constants.Padding.Left;
	
	local genderLabel = CreateLabel(self, false, left, top);
	genderLabel:SetText(Strings["target_gender"]);
	
	left = left + genderLabel:GetWidth();
	
	local genderSelectors =
	{
		{ Key = "Male_One", Image1 = Globals.Icons.Male, Image2 = nil },
		{ Key = "Female_One", Image1 = Globals.Icons.Female, Image2 = nil },
		{ Key = "Male_Many", Image1 = Globals.Icons.Male, Image2 = Globals.Icons.Male },
		{ Key = "Female_Many", Image1 = Globals.Icons.Female, Image2 = Globals.Icons.Female },
		{ Key = "Mixed", Image1 = Globals.Icons.Male, Image2 = Globals.Icons.Female }
	};
	
	for genderNumber = 1, 5 do
		local gender = genderSelectors[genderNumber];
		
		local selector = CreateRadioButton(self, "", "gender", left, top, Constants.Sizes.GenderIcon, Constants.Sizes.GenderIcon, gender.Image1, gender.Image2);
		selector.Selected = function (sender, args)
			self.SelectedGender = gender.Key;
			self:ShowTranslatedText(true);
		end
		
		left = left + Constants.Sizes.GenderIcon * 2 + Constants.Sizes.GenderIcon / 2;
		if gender.Image2 ~= nil then
			left = left + Constants.Sizes.GenderIcon;
		end
	end

	top = top + Constants.Sizes.GenderIcon + Constants.Padding.Lines;
	left = Constants.Padding.Left;
	
	local channelListLabel = CreateLabel(self, false, left, top);
	channelListLabel:SetText(Strings["output_channel"]);
	
	left = left + channelListLabel:GetWidth();
	
	self.ChannelList = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.ChannelList:AddItem(Strings["say"], Strings["cmd_say"]);
	self.ChannelList:AddItem(Strings["fellowship"], Strings["cmd_fellowship"]);
	self.ChannelList:AddItem(Strings["kinship"], Strings["cmd_kinship"]);
	self.ChannelList:AddItem(Strings["raid"], Strings["cmd_raid"]);
	self.ChannelList:AddItem(Strings["userchat1"], Strings["cmd_userchat1"]);
	self.ChannelList:AddItem(Strings["userchat2"], Strings["cmd_userchat2"]);
	self.ChannelList:AddItem(Strings["userchat3"], Strings["cmd_userchat3"]);
	self.ChannelList:AddItem(Strings["userchat4"], Strings["cmd_userchat4"]);
	self.ChannelList:AddItem(Strings["userchat5"], Strings["cmd_userchat5"]);
	self.ChannelList:AddItem(Strings["userchat6"], Strings["cmd_userchat6"]);
	self.ChannelList:AddItem(Strings["userchat7"], Strings["cmd_userchat7"]);
	self.ChannelList:AddItem(Strings["userchat8"], Strings["cmd_userchat8"]);
	self.ChannelList:SetSelectedValue(Settings.NeoKhuzdulWindow.DefaultChannel or Strings["cmd_say"], false);
	self.ChannelList.SelectedIndexChanged = function(sender, args)
		self:ShowTranslatedText(true);
	end
	
	top = top + Constants.Sizes.Dropdown.Height + Constants.Padding.Lines;
	left = Constants.Padding.Left;
	
	local filterLabel = CreateLabel(self, false, left, top);
	filterLabel:SetText(Strings["filter"]);
	
	left = left + channelListLabel:GetWidth();
		
	self.FilterBox = CreateTextBox(self, false);
	self.FilterBox:SetPosition(left, top);
	self.FilterBox:SetWidth(Constants.Sizes.TextBox);
	self.FilterBox.FocusGained = function (sender, args)
		self.FilterBox.SaveText = self.FilterBox:GetText();
		self.FilterBox:SetWantsUpdates(true);
	end
	self.FilterBox.FocusLost = function (sender, args)
		self.FilterBox:SetWantsUpdates(false);
	end
	self.FilterBox.Update = function (sender, args)
		local text = sender:GetText();
		if text ~= sender.SaveText then
			sender.SaveText = text;
			self:FilterPhrases(text);
		end
	end
	
	top = top + filterLabel:GetHeight() + Constants.Padding.Lines;
	left = Constants.Padding.Left;
	
	self.PhrasesListBox = PhrasesListBox(self);
	self.PhrasesListBox:SetPosition(left, top);
	self.PhrasesListBox.SelectedIndexChanged = function(sender, args)
		if args.Index > 0 then
			self.SelectedPhrase = args.Data;
			self.SelectedPhraseText = args.Text;
			
			local info = args.Data.Info;
			if info ~= nil then
				self.InfoBox:SetText(info[LANGUAGE] or info["EN"]);
			else
				self.InfoBox:SetText("");
			end
		else
			self.SelectedPhrase = nil;
			self.SelectedPhraseText = "";
			self.InfoBox:SetText("");
		end
		self:ShowTranslatedText(not self.FilterBox:GetWantsUpdates());
	end
	
	self.SortIcon = Turbine.UI.Control();
	self.SortIcon:SetParent(self);
	self.SortIcon:SetSize(Constants.Sizes.SortIcon.Width, Constants.Sizes.SortIcon.Height);
	self.SortIcon:SetTop(top - Constants.Sizes.SortIcon.Height - Constants.Padding.Lines);
	self.SortIcon.SetIcon = function(hover)
		local icon;
		
		if self.SortOrder == "Asc" then
			if hover then
				icon = Globals.Icons.SortedAsc_Hover;
			else
				icon = Globals.Icons.SortedAsc;
			end
		elseif self.SortOrder == "Desc" then
			if hover then
				icon = Globals.Icons.SortedDesc_Hover;
			else
				icon = Globals.Icons.SortedDesc;
			end
		else
			if hover then
				icon = Globals.Icons.SortedNone_Hover;
			else
				icon = Globals.Icons.SortedNone;
			end
		end
		self.SortIcon:SetBackground(icon);
	end
	self.SortIcon.SetIcon(false);
	self.SortIcon.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			if self.SortOrder == "Asc" then
				self.SortOrder = "Desc"
				self:SortPhrases();
			elseif self.SortOrder == "Desc" then
				self.SortOrder = nil;
				self:PopulatePhrases();
			else
				self.SortOrder = "Asc";
				self:SortPhrases();
			end
			self.SortIcon.SetIcon(true);
		end
	end	
	self.SortIcon.MouseEnter = function(sender,args)
		self.SortIcon.SetIcon(true);
	end
	self.SortIcon.MouseLeave = function(sender,args)
		self.SortIcon.SetIcon(false);
	end
	
	self.SortLabel = CreateLabel(self, false, 0, self.SortIcon:GetTop());
	self.SortLabel:SetSize(Constants.Sizes.SortLabel, self.SortIcon:GetHeight());
	self.SortLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.SortLabel:SetText(Strings["sort"]);
	
	self.InfoBox = CreateTextBox(self, true);
	self.InfoBox:SetPosition(left, top);
	self.InfoBox:SetHeight(Constants.Sizes.PhraseBox);
	self.InfoBox:SetEnabled(false);
	self.InfoBox:SetBackColor(Globals.Colours.Disabled);
	self.InfoBox:SetVisible(false);
	
	self.MoreInfoButton = CreateButton(self, 0);
	self.MoreInfoButton:SetText(Strings["show_info"]);
	self.MoreInfoButton.MouseClick = function(sender, args)
		if self.MoreInfoIsShown then
			self.MoreInfoButton:SetText(Strings["show_info"]);
		else
			self.MoreInfoButton:SetText(Strings["hide_info"]);
		end
		self.MoreInfoIsShown = not self.MoreInfoIsShown;
		self.InfoBox:SetVisible(self.MoreInfoIsShown);
		self:LayoutControls();
	end
	
	self.PlaceHolderLabel = CreateLabel(self, false, left, top);
	self.PlaceHolderLabel:SetText(Strings["placeholder_text"]);
	self.PlaceHolderLabel:SetForeColor(Globals.Colours.Disabled);
	
	self.PlaceHolderBox = CreateTextBox(self);
	self.PlaceHolderBox:SetWantsUpdates(false);
	self.PlaceHolderBox:SetEnabled(placeholderVisible);
	self.PlaceHolderBox:SetForeColor(Globals.Colours.Disabled);
	self.PlaceHolderBox:SetLeft(self.PlaceHolderLabel:GetLeft() + self.PlaceHolderLabel:GetWidth());
	self.PlaceHolderBox.FocusGained = function(sender, args)
		self.PlaceHolderBox:SetWantsUpdates(true);
	end
	self.PlaceHolderBox.FocusLost = function(sender, args)
		self.PlaceHolderBox:SetWantsUpdates(false);
		self:ShowTranslatedText(true);
	end
	self.PlaceHolderBox.Update = function(sender, args)
		local newText = self.PlaceHolderBox:GetText();
		if self.PlaceHolderText ~= newText then
			self.PlaceHolderText = newText;
			self:ShowTranslatedText(true);
		end
	end
	
	self.TranslationBox = CreateTextBox(self, true);
	self.TranslationBox:SetLeft(left);
	self.TranslationBox:SetHeight(Constants.Sizes.PhraseBox);
	self.TranslationBox:SetReadOnly(true);
	
	self.Shortcut = Turbine.UI.Lotro.Shortcut();
	self.Shortcut:SetType(Turbine.UI.Lotro.ShortcutType.Alias);
	self.Shortcut:SetData("/");
	
	self.Quickslot = Turbine.UI.Lotro.Quickslot();
	self.Quickslot:SetParent(self);
	self.Quickslot:SetSize(Constants.Sizes.ReadButton - 2, Constants.Sizes.ReadButton - 2);
	self.Quickslot:GetAllowDrop(false);
	self.Quickslot:SetVisible(false);
	self:SetShortcut();
	
	-- Fix for dragging short-cut
	self.Quickslot.DragDrop = function(sender, args)
		self:SetShortcut(); -- restore shortcut
	end
	
	self.QuickslotIcon = Turbine.UI.Control();
	self.QuickslotIcon:SetParent(self);
	self.QuickslotIcon:SetSize(Constants.Sizes.ReadButton, Constants.Sizes.ReadButton);
	self.QuickslotIcon:SetZOrder(self.Quickslot:GetZOrder() + 1);
	self.QuickslotIcon:SetMouseVisible(false);
	self.QuickslotIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut);
	self.QuickslotIcon.EnabledChanged = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut);
		else
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Inactive);
		end
	end
	self.QuickslotIcon:SetEnabled(false);
	
	self.HideTranslationCheckBox = CreateCheckBox(self, Strings["hide_translation"], left, 0);
	self.HideTranslationCheckBox:SetSize(Globals.Sizes.CheckBox.Width, Globals.Sizes.CheckBox.Height);
	self.HideTranslationCheckBox:SetChecked(Settings.NeoKhuzdulWindow.HideTranslation or false);
	self.HideTranslationCheckBox.CheckedChanged = function (sender, args)
		self:SetShortcutText();
	end
	
	self.Quickslot.MouseEnter = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Hover);
		end
	end
	self.Quickslot.MouseLeave = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut);
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self.UIHidden then
			if self:IsVisible() then
				Settings.NeoKhuzdulWindow.Visible = "yes";
			else
				Settings.NeoKhuzdulWindow.Visible = "no";
				self:SaveSettings(false);
			end
		end
		
		if self:IsVisible() then
			CheckWindowPosition(self);
		end
	end

	self:LayoutControls();
	self:PopulatePhrases();
end

function NeoKhuzdulWindow:LayoutControls()
	self.SortIcon:SetLeft(self:GetWidth() - Constants.Padding.Right - Constants.Sizes.SortIcon.Width);
	self.SortLabel:SetLeft(self:GetWidth() - Constants.Padding.Right - Constants.Sizes.SortIcon.Width - Constants.Sizes.SortLabel - Constants.Padding.Lines);

	self.MoreInfoButton:SetLeft(self:GetWidth() - Constants.Padding.Right - self.MoreInfoButton:GetWidth());
	self.QuickslotIcon:SetPosition((self:GetWidth() - self.QuickslotIcon:GetWidth()) / 2, self:GetHeight() - Constants.Padding.Bottom - self.QuickslotIcon:GetHeight());
	self.Quickslot:SetPosition(self.QuickslotIcon:GetLeft() + 1, self.QuickslotIcon:GetTop() + 1);
	
	self.HideTranslationCheckBox:SetTop(self.QuickslotIcon:GetTop());

	local width = self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right;
	
	self.TranslationBox:SetTop(self.Quickslot:GetTop() - Constants.Padding.Lines - self.TranslationBox:GetHeight());
	self.TranslationBox:SetWidth(width);

	self.PlaceHolderLabel:SetTop(self.TranslationBox:GetTop() - Constants.Padding.Lines - self.PlaceHolderLabel:GetHeight());
	self.PlaceHolderBox:SetTop(self.TranslationBox:GetTop() - Constants.Padding.Lines - self.PlaceHolderBox:GetHeight());
	self.MoreInfoButton:SetTop(self.PlaceHolderBox:GetTop());
	
	local adjustHeight = 0;

	if self.InfoBox:IsVisible() then
		adjustHeight = self.InfoBox:GetHeight() + Constants.Padding.Lines;
		self.InfoBox:SetWidth(width);
		self.InfoBox:SetTop(self.PlaceHolderBox:GetTop() - adjustHeight);
	end
	
	self.PhrasesListBox:SetSize(width, self.PlaceHolderBox:GetTop() - self.PhrasesListBox:GetTop() - Constants.Padding.Lines - adjustHeight);
end

function NeoKhuzdulWindow:Show()
	self:SetVisible(true);
	self:Activate();
end

function NeoKhuzdulWindow:Hide()
	if self:IsVisible() then
		self:SetVisible(false);
	end
end

function NeoKhuzdulWindow:Toggle()
	if self:IsVisible() then
		self:Hide();
	else
		self:Show();
	end
end

function NeoKhuzdulWindow:SaveSettings(showMessage)
	Settings.NeoKhuzdulWindow.Position.Top = tostring(self:GetTop());
	Settings.NeoKhuzdulWindow.Position.Left = tostring(self:GetLeft());
	Settings.NeoKhuzdulWindow.Size.Width = tostring(self:GetWidth());
	Settings.NeoKhuzdulWindow.Size.Height = tostring(self:GetHeight());
	Settings.NeoKhuzdulWindow.DefaultChannel = self.ChannelList:GetSelectedValue();
	Settings.NeoKhuzdulWindow.HideTranslation = self.HideTranslationCheckBox:IsChecked();
	
	Settings.ToggleWindow.Position.Left = tostring(toggleWindow:GetLeft());
	Settings.ToggleWindow.Position.Top = tostring(toggleWindow:GetTop());		
	
	NeoKhuzdulSave(Turbine.DataScope.Account, "NeoKhuzdulSettings", Settings);
end

function NeoKhuzdulWindow:PopulatePhrases()
	self.PhrasesListBox:ClearItems();
	self.DisplayedPhrases = {};
	
	local categoryKey = self.SelectedCategory;
	local keyword = self.FilterBox.SaveText or "";
	
	if categoryKey ~= nil then
		for phraseIndex, phraseData in ipairs(NeoKhuzdulDB.Phrases) do 
			if categoryKey == "" or phraseData.CategoryKey == categoryKey then
				table.insert(self.DisplayedPhrases, phraseData);
			end
		end
		
		self:SortPhrases();
	end
end

function NeoKhuzdulWindow:SortPhrases()
	if self.SortOrder ~= nil then
		table.sort(self.DisplayedPhrases, function(phraseAData, phraseBData)
			local phraseA = (phraseAData[LANGUAGE] or phraseAData["EN"]):gsub("___%s*", ""):gsub("^(.*)(%([^%)]*%)%s*)(.*)$", "%1%3%2");
			local phraseB = (phraseBData[LANGUAGE] or phraseBData["EN"]):gsub("___%s*", ""):gsub("^(.*)(%([^%)]*%)%s*)(.*)$", "%1%3%2");
			
			if self.SortOrder == "Asc" then
				return phraseA < phraseB;
			else
				return phraseA > phraseB;
			end
		end)
	end
	
	local keyword = self.FilterBox.SaveText or "";
	self:FilterPhrases(keyword, false);
end

function NeoKhuzdulWindow:FilterPhrases(keyword, reselectLine)
	if reselectLine == nil then
		reselectLine = true;
	end
	
	local currentPhrase = self.SelectedPhrase
	self.PhrasesListBox:ClearItems();
	
	local newIndex = 1;
	local reselectPhraseIndex = 0;
	for index, phraseData in ipairs(self.DisplayedPhrases) do 
		local phraseText = phraseData[LANGUAGE] or phraseData["EN"];
		if string.len(keyword) == 0 or phraseText:Contains(keyword) then
			self.PhrasesListBox:AddItem(phraseText, phraseData);
			if currentPhrase ~= nil and phraseData == currentPhrase then
				reselectPhraseIndex = newIndex;
			end
			newIndex = newIndex + 1;
		end
	end
	
	if reselectLine and reselectPhraseIndex > 0 then
		self.PhrasesListBox:SelectLine(reselectPhraseIndex);
	--else
	--	self:ShowTranslatedText();
	--	self.SelectedPoemId = nil;
	end
end

function NeoKhuzdulWindow:ShowTranslatedText(doFocus)
	local khuzdulText = nil;
	local phraseText = nil;
	local enableButton = false;
	local placeholderVisible = false;
	
	if self.LastText == nil then
		self.LastText = "";
	end
	
	if self.SelectedPhrase ~= nil and self.SelectedGender ~= nil then
		local phrase = self.SelectedPhrase;
		
		if self.SelectedGender == "Mixed" then
			khuzdulText = phrase["Mixed"] or phrase["Male_Many"] or phrase["All"];
		else
			khuzdulText = phrase[self.SelectedGender] or phrase["All"];
		end
		phraseText = (phrase[LANGUAGE] or phrase["EN"]);
		enableButton = true;

		if khuzdulText:find("___", 1, true) ~= nil then
			placeholderVisible = true;
			if self.PlaceHolderText ~= "" then
				khuzdulText = khuzdulText:gsub("___+", self.PlaceHolderText);
				phraseText = phraseText:gsub("___+", self.PlaceHolderText);
			else
				enableButton = false;
			end
		end
		
		self.TranslationBox:SetText(khuzdulText);
	else
		self.TranslationBox:SetText("");
	end
	
	if placeholderVisible then
		self.PlaceHolderLabel:SetForeColor(Globals.Colours.Label);
		self.PlaceHolderBox:SetForeColor(Globals.Colours.TextBox);
		if self.LastText ~= self.SelectedPhraseText or not self.PlaceHolderBox:IsEnabled() then
			self.PlaceHolderBox:SetEnabled(true);
			if doFocus then
				self.PlaceHolderBox:Focus();
			end
		end
	else
		self.PlaceHolderBox:SetEnabled(false);
		self.PlaceHolderLabel:SetForeColor(Globals.Colours.Disabled);
		self.PlaceHolderBox:SetForeColor(Globals.Colours.Disabled);
	end
	
	self.LastText = self.SelectedPhraseText;
	
	if enableButton then
		self.KhuzdulText = khuzdulText;
		self.PhraseText = phraseText;
		self.Quickslot:SetVisible(true);
		self.QuickslotIcon:SetEnabled(true);
		self:SetShortcutText();
	else
		self.Quickslot:SetVisible(false);
		self.QuickslotIcon:SetEnabled(false);
	end
end

function NeoKhuzdulWindow:SetShortcutText()
	if self.Quickslot:IsVisible() then
		if self.HideTranslationCheckBox:IsChecked() then
			self:SetShortcut(self.KhuzdulText);
		else
			self:SetShortcut(self.KhuzdulText .. " ((" .. self.PhraseText .. "))");
		end
	end
end

function NeoKhuzdulWindow:SetShortcut(text)
	if text ~= nil then
		self.Shortcut:SetData(self.ChannelList:GetSelectedValue() .. " " .. text);
	end
	self.Quickslot:SetShortcut(self.Shortcut);
end
